@@ -2,6 +2,7 @@
 
 import urllib
 import urllib2
+import cookielib
 from BeautifulSoup import BeautifulSoup
 from BeautifulSoup import BeautifulStoneSoup
 from BeautifulSoup import Comment
@@ -19,6 +20,7 @@ import uuid
 import os
 import re
 import locale
+import glob
 
 
 month_names = {"Jan": 1, "Feb": 2, "Mar": 3, "Apr": 4, "May": 5, "Jun": 6, \
@@ -30,6 +32,7 @@ zone_names = {"UT": "+0000", "GMT": "+0000", "EST": "-0500", "EDT": "-0400", \
 massage = [(re.compile('<([^<>\s]+)([^<>]*)/>'), \
     lambda x: "<" + x.group(1) + x.group(2) + "></" + x.group(1) + ">")]
 
+
 def parse_date(str):
     str = str.strip()
     parts = str.split(" ")
@@ -75,6 +78,24 @@ def parse_date(str):
     return (datetime(y, m, d, hh, mm, ss, ms), timedelta(minutes = dm, hours = dh))
 
 
+def create_url_opener():
+    filename = os.path.join(os.environ["HOME"], ".mozilla/microb/cookies.txt")
+    if not os.path.exists(filename):
+        files = glob.glob(os.path.join(os.environ["HOME"], ".mozilla/firefox/*default/cookies.txt"))
+        if len(files):
+            filename = files[0]
+        else:
+            filename = None
+    if filename:
+        cj = cookielib.MozillaCookieJar()
+        cj.load(filename)
+        return urllib2.build_opener(urllib2.HTTPCookieProcessor(cj))
+    return urllib2.build_opener()
+
+
+urlopener = create_url_opener()
+
+
 def quote_url_path(url):
     p = urlsplit(url)
     return urlunsplit((p[0], p[1], urllib.quote(p[2]), p[3], p[4]))
@@ -109,6 +130,38 @@ def get_page_base(page, html):
     return page.geturl()
 
 
+def get_page_html_only(url, **kwargs):
+    page = urlopener.open(urllib2.Request(url=quote_url_path(url), **kwargs))
+    html = BeautifulSoup(page, fromEncoding = get_page_encoding(page))
+    base = get_page_base(page, html)
+    return (page, html, base)
+
+
+refresh_reqex = re.compile("refresh", re.IGNORECASE)
+
+
+def try_skip_banner(page, html, base, **kwargs):
+    refresh = None
+    if page.info().has_key("Refresh"):
+        refresh = page.info()["Refresh"]
+    else:
+        tag = html.find(name = "meta", attrs = {"http-equiv": refresh_reqex})
+        if tag:
+            refresh = tag["content"]
+    if refresh:
+        new_url = page.geturl()
+        ind = refresh.lower().find("url=")
+        if ind >= 0:
+            new_url = urljoin(base, refresh[ind + 4:])
+        return get_page_html_only(new_url, **kwargs)
+    return (page, html, base)
+
+
+def get_page_html(url, **kwargs):
+    (page, html, base) = get_page_html_only(url, **kwargs)
+    #(page, html, base) = try_skip_banner(page, html, base, **kwargs)
+    return (html, base, page.geturl())
+
 def get_feed_title(xml):
     chan = xml.find(["channel", "feed"])
     if chan:
@@ -119,7 +172,7 @@ def get_feed_title(xml):
 
 
 def discover_feed(url):
-    page = urllib2.urlopen(url)
+    page = urlopener.open(url)
     icon_url = None
     content_type = None
     if page.info().has_key("Content-Type"):
@@ -135,7 +188,7 @@ def discover_feed(url):
             rss_tag = html.find(name = "link", attrs = {"rel": "alternate", "type": "application/atom+xml"})
         if rss_tag:
             url = urljoin(base, rss_tag["href"])
-            page = urllib2.urlopen(url)
+            page = urlopener.open(url)
             content_type = page.info()["Content-Type"]
     xml = BeautifulStoneSoup(page, markupMassage = massage)
     title = get_feed_title(xml)
@@ -155,7 +208,7 @@ def discover_feed(url):
             if html_tag:
                 html_url = html_tag.string
         if html_url:
-            page = urllib2.urlopen(html_url)
+            page = urlopener.open(html_url)
             html = BeautifulSoup(page, fromEncoding = get_page_encoding(page))
             base = get_page_base(page, html)
             icon_tag = html.find(name = "link", attrs = {"rel": re.compile("icon")})
@@ -228,36 +281,33 @@ class Item:
         return cache_path
 
 
-    def __find_print_version(self, page, html):
+    def __find_print_version(self, html, base, referer):
         try:
             text = html.find(text=lambda t: t.strip().lower() in self.__print_version_keywords)
             if text:
                 a = text.findParent(lambda t: t.name == "a" and t.has_key("href"))
                 if a:
-                    url = urljoin(get_page_base(page, html), a["href"])
+                    url = urljoin(base, a["href"])
                     self.__report("downloading", url)
-                    new_page = urllib2.urlopen(urllib2.Request(url=quote_url_path(url), headers={"Referer": page.geturl()}))
-                    new_html = BeautifulSoup(new_page, fromEncoding = get_page_encoding(new_page))
-                    return (new_page, new_html, new_html.body)
+                    (new_html, new_base, real_url) = get_page_html(url, headers={"Referer": referer})
+                    return (new_html, new_base, new_html.body)
         except Exception, ex:
             self.__report("error", ex)
-        return (page, html, None)
+        return (html, base, None)
 
 
     def __extract_tag(self):
         self.__report("downloading", self.__url)
-        page = urllib2.urlopen(quote_url_path(self.__url))
-        html = BeautifulSoup(page, fromEncoding = get_page_encoding(page))
+        (html, self.__base, real_url) = get_page_html(self.__url)
         if not self.__as_is:
             if self.__content_kwargs:
                 self.__tag = html.body.find(**self.__content_kwargs)
             elif self.__try_use_print_version:
-                (page, html, self.__tag) = self.__find_print_version(page, html)
+                (html, self.__base, self.__tag) = self.__find_print_version(html, self.__base, real_url)
             if not self.__tag:
                 self.__tag = self.__find_the_tag(html.body)
         if not self.__tag:
             self.__tag = html.body
-        self.__base = get_page_base(page, html)
         title = html.find("title")
         if title:
             self.title = title.string.strip()
@@ -408,6 +458,9 @@ class Feed:
 
 
     def __get_item_link(self, item):
+        if item.guid:
+            if not item.guid.has_key("isPermaLink") or item.guid["isPermaLink"].lower() == "true":
+                return item.guid.string
         if item.link:
             if item.link.has_key("href"):
                 return item.link["href"] #Atom uses href to store links
@@ -516,7 +569,7 @@ class Feed:
         result = 0
         try:
             self.__report("downloading", self.__url)
-            page = urllib2.urlopen(quote_url_path(self.__url))
+            page = urlopener.open(quote_url_path(self.__url))
             self._real_url = page.geturl()
             xml = BeautifulStoneSoup(page, markupMassage = massage)
             if not self.__title:
