@@ -22,7 +22,7 @@ gtk.gdk.threads_init()
 #This is to be able to start Feed Circuit on PC
 try:
     import osso
-    osso_context = osso.Context("org.maemo.feedcircuit", "0.6.2", False)
+    osso_context = osso.Context("org.maemo.feedcircuit", "0.6.3", False)
 except ImportError:
     pass
 
@@ -121,6 +121,12 @@ class ConfigParserEx(ConfigParser.SafeConfigParser):
         return default
 
 
+    def set(self, section, option, value):
+        val = value
+        if val:
+            val = val.replace("%", "%%")
+        ConfigParser.SafeConfigParser.set(self, section, option, val)
+
 class FeedConfig:
 
 
@@ -131,7 +137,7 @@ class FeedConfig:
 
     def __init__(self, url, name, mode, days_to_keep,
         include, exclude, ignore_print_version, icon_filename,
-        message_count, as_is, content_tag, id = None):
+        message_count, as_is, content_tag, print_version_link, id = None):
         self.id = id
         if not self.id:
             self.id = uuid.uuid1().hex
@@ -146,6 +152,7 @@ class FeedConfig:
         self.message_count = message_count
         self.as_is = as_is
         self.content_tag = content_tag
+        self.print_version_link = print_version_link
         self.icon = None
         self.iter = None
         self.ord_num = 0
@@ -176,6 +183,11 @@ class FeedConfig:
         return self.name
 
 
+    def get_print_version_keywords(self):
+        if self.print_version_link:
+            return [self.print_version_link, ]
+
+
     def get_content_kwargs(self):
         result = {}
         if self.content_tag:
@@ -284,6 +296,7 @@ class FeedListModel(gtk.GenericTreeModel):
                 parser.getint_default(section, "message count", 0),
                 parser.getboolean_default(section, "as is", False),
                 parser.get_default(section, "content tag", ""),
+                parser.get_default(section, "print version link", ""),
                 section)
             feed_config.ord_num = parser.getint_default(section, "order number", 0)
             self.list.append(feed_config)
@@ -294,7 +307,7 @@ class FeedListModel(gtk.GenericTreeModel):
     def save_config(self):
         f = open(os.path.join(self.config_path, "config"), "w")
         try:
-            parser = ConfigParser.SafeConfigParser()
+            parser = ConfigParserEx()
             i = 0
             for feed_config in self.list:
                 parser.add_section(feed_config.id)
@@ -309,6 +322,7 @@ class FeedListModel(gtk.GenericTreeModel):
                 parser.set(feed_config.id, "message count", str(feed_config.message_count))
                 parser.set(feed_config.id, "as is", str(feed_config.as_is))
                 parser.set(feed_config.id, "content tag", feed_config.content_tag)
+                parser.set(feed_config.id, "print version link", feed_config.print_version_link)
                 parser.set(feed_config.id, "order number", str(i))
                 i += 1
             parser.write(f)
@@ -377,7 +391,7 @@ class ExtraWindow(gtk.Dialog):
         self.include_entry.set_width_chars(24)
         self.exclude_entry = gtk.Entry()
         table = gtk.Table(rows = 4, columns = 2)
-        texts = [_("Days to keep"), _("Include pattern"), _("Exclude pattern") ]
+        texts = [_("Days to keep"), _("Include urls"), _("Exclude urls") ]
         for i in range(0, len(texts)):
             label = gtk.Label(texts[i])
             label.set_alignment(1, 1)
@@ -400,27 +414,41 @@ class ContentWindow(gtk.Dialog):
         self.set_title(_("Content settings"))
         self.as_is_button = gtk.CheckButton()
         self.as_is_button.connect("toggled", self.as_is_changed)
-        self.ignore_print_version_button = gtk.CheckButton()
         self.content_tag_entry = gtk.Entry()
         self.content_tag_entry.set_width_chars(24)
-        table = gtk.Table(rows = 3, columns = 2)
-        texts = [_("Use as is"), _("Ignore print version"), _("Content tag") ]
+        self.content_tag_entry.connect("changed", self.content_tag_entry_changed)
+        self.ignore_print_version_button = gtk.CheckButton()
+        self.ignore_print_version_button.connect("toggled", self.ignore_print_version_changed)
+        self.print_version_link_entry = gtk.Entry()
+        table = gtk.Table(rows = 4, columns = 2)
+        texts = [_("Use as is"), _("Content tag"), _("Ignore print version"), _("Print version link") ]
         for i in range(0, len(texts)):
             label = gtk.Label(texts[i])
             label.set_alignment(1, 1)
             table.attach(label, 0, 1, i, i + 1, xpadding = 3, ypadding = 3)
         table.attach(self.as_is_button, 1, 2, 0, 1, xpadding = 3, ypadding = 3)
-        table.attach(self.ignore_print_version_button, 1, 2, 1, 2, xpadding = 3, ypadding = 3)
-        table.attach(self.content_tag_entry, 1, 2, 2, 3, xpadding = 3, ypadding = 3)
+        table.attach(self.content_tag_entry, 1, 2, 1, 2, xpadding = 3, ypadding = 3)
+        table.attach(self.ignore_print_version_button, 1, 2, 2, 3, xpadding = 3, ypadding = 3)
+        table.attach(self.print_version_link_entry, 1, 2, 3, 4, xpadding = 3, ypadding = 3)
         self.vbox.add(table)
         self.show_all()
 
 
     def as_is_changed(self, source):
         self.ignore_print_version_button.set_sensitive(not self.as_is_button.get_active())
+        self.print_version_link_entry.set_sensitive(not self.as_is_button.get_active())
         self.content_tag_entry.set_sensitive(not self.as_is_button.get_active())
 
 
+    def ignore_print_version_changed(self, source):
+        self.print_version_link_entry.set_sensitive(not self.ignore_print_version_button.get_active())
+
+
+    def content_tag_entry_changed(self, source):
+        self.ignore_print_version_button.set_sensitive(len(self.content_tag_entry.get_text()) == 0)
+        self.print_version_link_entry.set_sensitive(len(self.content_tag_entry.get_text()) == 0)
+
+
 class EditWindow(gtk.Dialog):
 
 
@@ -437,6 +465,7 @@ class EditWindow(gtk.Dialog):
         self.ignore_print_version = feed_config.ignore_print_version
         self.as_is = feed_config.as_is
         self.content_tag = feed_config.content_tag
+        self.print_version_link = feed_config.print_version_link
         self.url_entry = gtk.Entry()
         self.url_entry.set_width_chars(32)
         self.url_entry.set_text(feed_config.url)
@@ -453,7 +482,7 @@ class EditWindow(gtk.Dialog):
         self.mode_combo.set_active(feed_config.mode)
         hbox.pack_start(self.mode_combo)
         hbox.pack_start(self.content_button, expand = False)
-        self.extra_button = gtk.Button(_("Extra settings"))
+        self.extra_button = gtk.Button(_("More"))
         self.extra_button.connect("clicked", self.extra_clicked)
         table = gtk.Table(rows = 4, columns = 2)
         texts = [_("Url"), _("Name"), _("Content")]
@@ -485,6 +514,7 @@ class EditWindow(gtk.Dialog):
         self.feed_config.as_is = self.as_is
         self.feed_config.ignore_print_version = self.ignore_print_version
         self.feed_config.content_tag = self.content_tag
+        self.feed_config.print_version_link = self.print_version_link
 
 
     def extra_clicked(self, source):
@@ -504,10 +534,12 @@ class EditWindow(gtk.Dialog):
         cnt_window.as_is_button.set_active(self.as_is)
         cnt_window.ignore_print_version_button.set_active(self.ignore_print_version)
         cnt_window.content_tag_entry.set_text(self.content_tag)
+        cnt_window.print_version_link_entry.set_text(self.print_version_link)
         if cnt_window.run() == gtk.RESPONSE_ACCEPT:
             self.as_is = cnt_window.as_is_button.get_active()
             self.ignore_print_version = cnt_window.ignore_print_version_button.get_active()
             self.content_tag = cnt_window.content_tag_entry.get_text()
+            self.print_version_link = cnt_window.print_version_link_entry.get_text()
         cnt_window.destroy()
 
 
@@ -703,7 +735,8 @@ class Program(hildon.Program):
                     inline_items = feed_config.mode == FeedConfig.mode_inline,
                     cache_items = feed_config.mode == FeedConfig.mode_cache,
                     include = feed_config.include, exclude = feed_config.exclude,
-                    as_is = feed_config.as_is, content_kwargs = feed_config.get_content_kwargs())
+                    as_is = feed_config.as_is, content_kwargs = feed_config.get_content_kwargs(),
+                    print_version_keywords = feed_config.get_print_version_keywords())
                 new_item_count = feed.update()
                 gobject.idle_add(self.feed_updated, feed_config, new_item_count)
         finally:
@@ -761,7 +794,7 @@ class Program(hildon.Program):
                     urllib.urlretrieve(icon_url, os.path.join(create_config_dir(), icon_filename))
             except Exception, ex:
                 pass
-            feed_config =FeedConfig(url, title, FeedConfig.mode_ommit, 7, "", "", False, icon_filename, 0, False, "")
+            feed_config =FeedConfig(url, title, FeedConfig.mode_ommit, 7, "", "", False, icon_filename, 0, False, "", "")
             edit_window = EditWindow(self.__window, feed_config)
             if edit_window.run() == gtk.RESPONSE_ACCEPT:
                 edit_window.update_feed()
@@ -795,7 +828,7 @@ class Program(hildon.Program):
     def about(self, source):
         dialog = gtk.AboutDialog()
         dialog.set_name("FeedCircuit")
-        dialog.set_version("0.6.2")
+        dialog.set_version("0.6.3")
         dialog.set_website("http://feedcircuit.garage.maemo.org")
         dialog.set_authors([_("George Kibardin")])
         dialog.run()
